unit IRQThreadUnit;

interface

uses
  Classes, Windows, MainUnit;

type
  TIRQThread = class(TThread)
  public
    COSPPIs: LongWord;
    CardNum: Integer;
    IRQCount: Integer;
  private
    COSData: TCOSData;
    LastError: LongWord;

    procedure ReportIRQ;
    function WaitForIRQType: LongWord;
    procedure ReportError;
  protected
    procedure Execute; override;
  end;

implementation

uses
  SysUtils, AIOWDM;

procedure TIRQThread.Execute;
begin
  SetLength(COSData, COSPPIs);
  while WaitForIRQType <> 0 do
    Synchronize(ReportIRQ)
  ;
  LastError := GetLastError;
  Synchronize(ReportError);
end;

procedure TIRQThread.ReportError;
begin
  if LastError = ERROR_OPERATION_ABORTED then
    MainForm.AddLine(CardNum, 'IRQ request aborted.')
  else if LastError = ERROR_INVALID_FUNCTION then
    MainForm.AddLine(CardNum, 'Error: this card already has a pending IRQ request. IRQ request aborted.')
  else
    MainForm.AddLine(CardNum, 'Some error occurred in the driver. IRQ request aborted.')
  ;
end;

procedure TIRQThread.ReportIRQ;
begin
  if COSPPIs = 0 then
    MainForm.ReceiveGenIRQ(CardNum)
  else
    MainForm.ReceiveCOSData(CardNum, COSData)
  ;
end;

function TIRQThread.WaitForIRQType: LongWord;
var
  tempword:word;
begin
  begin
    Result := WaitForIRQ(CardNum);
    tempword := RelInport(CardNum, $10); // read which bits generated CoS IRQ
    COSData[0].a := tempword and 255;
    COSData[0].b := tempword div 256;
    inc(IRQCount);
    COSData[0].c := IRQCount;
    RelOutPort(CardNum, $10, $0000); // Clear all IRQs

  end;
end;

end.
